DEFAULT_VRF = {
    "arista": {"eos": "default"},
    "azure": {None: ""},
    "brocade": {"fastiron": "default-vrf"},
    "checkpoint": {"gaia": "0", "gaia-embedded": ""},
    "cisco": {
        "ios": "",
        "ios-xe": "",
        "sg": "",
        "nx-os": "default",
        "ios-xr": "default",
        "aci": "default",
        "asa": "",
        "wlc-air": "",
        "ftd": "",
        "lap": "",
        "meraki": "",
        "apic": "",
        "encs": "",
        "viptela": "0",
    },
    "dell": {
        "ftos": "",
        "powerconnect": "",
        "os10": "",
    },
    "extreme": {
        "boss": "",
        "enterasys": "",
        "exos": "VR-Default",
        "voss": "GlobalRouter",
    },
    "f5": {"big-ip": "0"},
    "fortinet": {"fortigate": "", "fortiswitch": ""},
    "frr": {None: "default"},
    "fs": {"fsos": ""},
    "hp": {
        "aruba": "",
        "arubasw": "",
        "comware": "",
        "aruba-lap": "",
        "comware-lap": "",
        "3com": "",
        "aruba-iap": "",
        "arubacx": "default",
    },
    "huawei": {"vrp": ""},
    "juniper": {
        "junos": "master",
        "mist": "master",
    },
    "paloalto": {None: ""},
    "quagga": {"vrp": "0"},
    "riverbed": {"steelhead": ""},
    "ruckus": {"vsz": ""},
    "mikrotik": {"routeros": "main"},
    "aws": {"vrp": "ec2"},
    "gcp": {None: ""},
    "versa": {"vos": "global"},
    "silverpeak": {"unity": ""},
    "vmware": {"nsx-t": "global", "velocloud": "Global Segment"},
    "forcepoint": {"ngfw": "", "smc": ""},
    "alcatel": {"aos": "default"},
    "opengear": {"og": "", "og-om": ""},
    "stormshield": {"sn": ""},
    "nokia": {"timos": "Base"},
    "citrix": {"adc": "0"},
}
